#!/usr/bin/env python
# -*- coding: utf-8 -*-
# test_us.py
"""Tests for academic_us module."""
import os

from context import academic_us


def test_filenotfound():
    try:
        os.remove('./persontitles/data/academic_us.txt')
    except FileNotFoundError:
        pass
    ACADEMIC = academic_us.degrees_us()
    assert isinstance(ACADEMIC, set)


def test_academic_is_set():

    ACADEMIC = academic_us.degrees_us()
    assert isinstance(ACADEMIC, set)


def test_degree_in_list():

    ACADEMIC = academic_us.degrees_us()
    assert 'B.F&TV.' in ACADEMIC
    assert 'B.S.-C.I.E.' in ACADEMIC
    assert 'Dr. e. h' not in ACADEMIC
    assert 'B.F&amp;TV.' not in ACADEMIC



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# academic_german.py
"""Collection of German academic degrees combining wiki and drtitel."""
import os
import sys
import re

PACKAGE_PARENT = '..'
SCRIPT_DIR = os.path.dirname(
    os.path.realpath(os.path.join(os.getcwd(),
    os.path.expanduser(__file__))),
)  # isort:skip # noqa # pylint: disable=wrong-import-position
sys.path.append(
    os.path.normpath(os.path.join(SCRIPT_DIR, PACKAGE_PARENT)),
)  # isort: skip # noqa # pylint: disable=wrong-import-position

from persontitles.academic_german_wiki import degrees_ger_wiki  # noqa
from persontitles.academic_german_drtitel import degrees_ger_drtitel  # noqa


def degrees_ger() -> list:
    try:
        with open('./src/persontitles/data/academic_german.txt', mode='r', encoding='utf-8') as fin:  # noqa
            DEGREES = fin.read().split('\n')
    except FileNotFoundError:
        DEGREES_WIKI = set(degrees_ger_wiki())
        DEGREES_DRTITEL = set(degrees_ger_drtitel())
        DEGREES = [dgr for dgr in DEGREES_WIKI | DEGREES_DRTITEL]

        with open('./src/persontitles/data/academic_german.txt', mode='a', encoding='utf-8') as fout:  # noqa
            fout.write('\n'.join(item for item in DEGREES))

    return DEGREES


def no_empty_space_in_degree(degrees):
    """
    Because there is no coherent writing of German degrees and some are written
    with empty spaces and some without, all degrees that potentially could
    have a space/s or other way round will be replicated with/without empty
    space/s.
    To make sure that writing a degree without empty spaces is covered, all
    degrees get reduced of their whitespace.
    """

    degrees_wo_empty_space = []
    for degree in degrees:
        degree = re.sub(r'\. ', '.', degree)
        degrees_wo_empty_space.append(degree)

    return degrees_wo_empty_space


def add_empty_space(degrees) -> list:
    """
    A degree like "Dipl.agr.biol." has no empty spaces between the grade of
    the degree (Dipl.) and its specification (agr. biol.). To make sure that
    both ways of writing are covered, the degree will be replicated as "Dipl.
    agr. biol.".
    """

    degrees_w_empty_space = []
    for degree in degrees:
        degree_w_space = ''
        dot_counter = 1
        max = len(degree) - 1
        for i, ch in enumerate(degree):
            if ch == '.' and i == max:
                if degree_w_space != '':
                    degrees_w_empty_space.append(degree_w_space.strip())
                else:
                    degrees_w_empty_space.append(degree.strip())
            elif ch == '.' and i < max:
                if degree[i + 1] not in ['-', ')']:
                    if degree_w_space == '':
                        degree_w_space = degree[:i + 1] + ' ' + degree[i + 1:]
                        dot_counter += 1
                    else:
                        degree_w_space = degree_w_space[:i+dot_counter] + ' ' + degree_w_space[i+dot_counter:]  # noqa
                        dot_counter += 1
            elif i == max:
                if degree_w_space != '':
                    degrees_w_empty_space.append(degree_w_space.strip())
                else:
                    degrees_w_empty_space.append(degree.strip())

    return degrees_w_empty_space


def german_abbrevs(DEGREES) -> list:
    """
    Because the aim of this module is to find the compounds of a person's name
    or degree or even peer title, the fact that German degrees are more often
    than not written with empty spaces between the degree and its specification
    makes it necessary to collect both those solitary specs like "rer.",
    "nat.", "oec.", and "med." (there are more), and also the degrees like
    "Dr." or "Dipl.".
    """

    degrees = add_empty_space(DEGREES)
    abbrevs = []

    for degree in degrees:
        elements = degree.split(' ')
        for element in elements:
            if element not in abbrevs:
                abbrevs.append(element)

    abbrevs = [element for element in abbrevs if element.strip()]
    return abbrevs


if __name__ == '__main__':
    DEGREES_WIKI = set(degrees_ger_wiki())
    DEGREES_DRTITEL = set(degrees_ger_drtitel())
    DEGREES = [dgr for dgr in DEGREES_WIKI | DEGREES_DRTITEL]
#    for i, degree in enumerate(sorted(DEGREES)):
#        print(i, degree)
    print()
    print('Count degrees from wiki:', len(DEGREES_WIKI))
    print('Count degrees from drtitel:', len(DEGREES_DRTITEL))

    print('Common degrees from both sets:', len(DEGREES_WIKI & DEGREES_DRTITEL))  # noqa
    print('Degrees only from wiki:', len(DEGREES_WIKI - DEGREES_DRTITEL))
    print('Degrees only from drtitel:', len(DEGREES_DRTITEL - DEGREES_WIKI))

    print('Sum of degrees of both sets:', len(DEGREES))

    degrees_wo = no_empty_space_in_degree(DEGREES)
    degrees_w = add_empty_space(degrees_wo)

    abbrevs = german_abbrevs(DEGREES)
    print('Number of abbreviations:', len(abbrevs))

    for degree in sorted(DEGREES):
        print(degree)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# salutations.py
"""Collection of salutations like "Mr." or "Frau"."""


def salutations() -> list:
    with open('./persontitles/resources/salutations.txt', mode='r', encoding='utf-8') as fin:  # noqa
        SALUTATIONS = [salut.strip() for salut in fin.read().split('\n') if salut.strip()]  # noqa

    return SALUTATIONS


if __name__ == '__main__':
    SALUTATIONS = salutations()
    for i, salut in enumerate(SALUTATIONS):
        print(i, salut)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# academic_german.py
"""Collection of German academic degrees as found on www.drtitel.de."""
import bs4
import re
import requests
import unicodedata
from bs4 import BeautifulSoup


def degrees_ger_drtitel() -> list:
    try:
        with open('./src/persontitles/data/academic_german_drtitel.txt', mode='r', encoding='utf-8') as fin:  # noqa
            DEGREES_DRTITEL = fin.read().split('\n')
    except FileNotFoundError:
        DEGREES_DRTITEL = dr_degrees()
        with open('./src/persontitles/data/academic_german_drtitel.txt', mode='a', encoding='utf-8') as fout:  # noqa
            fout.write('\n'.join(item for item in DEGREES_DRTITEL))

    return DEGREES_DRTITEL


def dr_degrees() -> list:
    soup = get_soup()
    lines = get_less_lines(soup)
    lines, degrees = get_part_of_degrees(lines)
    lines, degrees = get_colon_part_of_degrees(lines, degrees)
    lines, degrees = get_FH_part_of_degrees(lines, degrees)
    lines = get_splitted_degrees(lines)
    lines, degrees = get_degrees_with_dots(lines, degrees)
    lines, degrees = get_degrees_w_abbr(lines, degrees)
    lines, degrees = get_last_degrees(lines, degrees)
    degrees = declutter_degrees(degrees)
    degrees = normalize_degrees(degrees)

    return degrees


def get_soup() -> bs4.element.NavigableString:
    data = requests.get('https://www.drtitel.de/akademische-grade/')
    soup = BeautifulSoup(data.text, 'lxml')

    return soup


def get_less_lines(soup):
    lines = []
    for li in soup.find_all('li'):
        lines.append(li.get_text(strip=True))

    counter = 0
    for i, line in enumerate(lines):
        if line.startswith('B.A.'):
            counter = i
            break
    lines = lines[counter:]

    counter = 0
    for i, line in enumerate(lines):
        if line.startswith('DDr.'):
            counter = i
            break
    lines = lines[:counter + 1]

    print('length lines:', len(lines))
    return lines


def get_part_of_degrees(lines):
    degrees = []
    lines_before = lines[:]
    for line in lines_before:
        if line.startswith('…'):
            degree = line[2:]
            degrees.append(degree.strip())
            lines.remove(line)

    print('length degrees, lines:', len(degrees), len(lines))
    return lines, degrees


def get_colon_part_of_degrees(lines: list, degrees: list):
    lines_ = lines[:]
    for i, line in enumerate(lines_):
        # print(i, line.strip())
        if line.endswith('):'):
            degree = line.split('(')[-1][:-2]
            degrees.append(degree)
            lines.remove(line)

    print('length degrees, lines:', len(degrees), len(lines))
    return lines, degrees


def get_FH_part_of_degrees(lines, degrees):
    lines_ = lines[:]
    for i, line in enumerate(lines_):
        # print(i, line)
        if line.startswith('Dipl.') and '(FH)' in line:
            degree = line.split('(FH)')[0] + ' (FH)'
            degrees.append(degree)
            lines.remove(line)
        elif line.startswith('Dipl.') and '(DH)' in line:
            degree = line.split('(DH)')[0] + ' (DH)'
            degrees.append(degree)
            lines.remove(line)
        elif line.startswith('Dipl.') and '(t.o.)' in line:
            degree = line.split('(t.o.)')[0] + ' (t.o.)'
            degrees.append(degree)
            lines.remove(line)
    print('length degrees, lines:', len(degrees), len(lines))
    return lines, degrees


def get_splitted_degrees(lines):
    lines_ = lines[:]
    for i, line in enumerate(lines_):
        if '/' in line:
            splits = line.split('/')
            for split in splits:
                lines.append(split)
            lines.remove(line)
        elif 'oder' in line:
            splits = line.split('oder')
            for split in splits:
                lines.append(split)
            lines.remove(line)
        elif 'bzw.' in line:
            splits = line.split('bzw.')
            for split in splits:
                lines.append(split)
            lines.remove(line)

    print('length lines:', len(lines))

    return lines


def get_degrees_with_dots(lines, degrees):
    lines_ = lines[:]
    for i, line in enumerate(lines_):
        abbrevs = line.split(' ')
        if '.' in abbrevs[0] and len(abbrevs) > 1:
            if '.' in abbrevs[1]:
                pass
            elif 'Diplom' in line and '+ Fachrichtung' not in line:
                degree = line.split('Diplom')[0].strip()
                degrees.append(degree)
                lines.remove(line)
            elif '(engl.' in line:
                degree = line.split('(engl.')[0].strip()
                degrees.append(degree)
                lines.remove(line)
            else:
                degrees.append(abbrevs[0])
                lines.remove(line)
        elif '.' in abbrevs[0]:
            degrees.append(abbrevs[0])
            lines.remove(line)

    print('length degrees, lines:', len(degrees), len(lines))
    return lines, degrees


def get_degrees_w_abbr(lines, degrees):
    lines_ = lines[:]
    for i, line in enumerate(lines_):
        abbrevs = line.split(' ')
        degree = ''
        for abbr in abbrevs:
            if '.' not in abbr:
                if abbr.strip() in ['in', 'et', 'pol'] and degree != '':  # noqa
                    degree = degree + abbr + ' '
                elif degree != '':
                    degrees.append(degree)
                    lines.remove(line)
                    break
            elif 'DDr.' in abbr:
                degrees.append(abbr)
                degree = ''
            elif '.' in abbr:
                degree = degree + abbr + ' '

    print('length degrees, lines:', len(degrees), len(lines))
    return lines, degrees


def get_last_degrees(lines, degrees):
    lines_ = lines[:]
    for i, line in enumerate(lines_):
        if line.endswith(':'):
            degrees.append(line[:-1])
            lines.remove(line)
        elif line.strip().startswith('Dipl'):
            degrees.append(line)
            lines.remove(line)
        elif 'MLE' in line:
            degrees.append('MLE')
            lines.remove(line)
        elif 'BBA' in line:
            degrees.append('BBA')
            lines.remove(line)
        elif 'Master' in line:
            degrees.append(line)
            lines.remove(line)

    print('length degrees, lines:', len(degrees), len(lines))
    return lines, degrees


def declutter_degrees(degrees) -> list:
    decluttered_degrees = []
    for degree in degrees:
        while '  ' in degree:
            degree = re.sub('  ', ' ', degree)
        degree = degree.strip()
        if degree.endswith('):'):
            degree = degree[:-2]
        elif degree.endswith(':'):
            degree = degree.strip()[:-1]
        elif '+ Fachrichtung' in degree:
            continue
        if degree.strip().startswith('('):
            degree = degree[1:]
        decluttered_degrees.append(degree.strip())

    return decluttered_degrees


def normalize_degrees(degrees) -> list:
    normalized_degrees = []
    for degree in degrees:
        degree = unicodedata.normalize('NFKD', degree.strip())
        degree = degree.strip()
        if degree not in normalized_degrees:
            normalized_degrees.append(degree)

    return normalized_degrees


if __name__ == '__main__':
    degrees = degrees_ger_drtitel()
    for i, degree in enumerate(degrees):
        print(i, degree)
